/// @file XMLStruct.cpp 
/// @brief  
/// @author semmal 
/// @date 2008-12-18 

#include <vector>

#include "XMLStruct.h"

const char * TYPE_BOOL = "bool";
const char * TYPE_CHAR = "char";
const char * TYPE_UCHAR = "uchar";
const char * TYPE_SHORT = "short";
const char * TYPE_USHORT = "ushort";
const char * TYPE_INT = "int";
const char * TYPE_UINT = "uint";
const char * TYPE_LONG = "long";
const char * TYPE_ULONG = "ulong";
const char * TYPE_LONGLONG = "longlong";
const char * TYPE_ULONGLONG = "ulonglong";
const char * TYPE_FLOAT = "float";
const char * TYPE_DOUBLE = "double";
const char * TYPE_LONGDOUBLE = "longdouble";

#define ADDMEMBER(ttype,tname) \
					MemberInfo member;\
					member.type(tname);\
					member.size(sizeof(ttype));\
					info.members[datatype.text()] = member;\
					s += member.size();\
					ms.push_back(&info.members[datatype.text()]);


void MemberInfo::set(bool value)
{
	memcpy(buffer(),&value,sizeof(bool));
}

void MemberInfo::set(char value)
{
	memcpy(buffer(),&value,sizeof(char));
}

void MemberInfo::set(unsigned char value)
{
	memcpy(buffer(),&value,sizeof(unsigned char));
}

void MemberInfo::set(short value)
{
	memcpy(buffer(),&value,sizeof(short));
}

void MemberInfo::set(unsigned short value)
{
	memcpy(buffer(),&value,sizeof(unsigned short));
}

void MemberInfo::set(int value)
{
	memcpy(buffer(),&value,sizeof(int));
}

void MemberInfo::set(unsigned int value)
{
	memcpy(buffer(),&value,sizeof(unsigned int));
}

void MemberInfo::set(long value)
{
	memcpy(buffer(),&value,sizeof(long));
}

void MemberInfo::set(unsigned long value)
{
	memcpy(buffer(),&value,sizeof(unsigned long));
}

void MemberInfo::set(long long value)
{
	memcpy(buffer(),&value,sizeof(long long));
}

void MemberInfo::set(unsigned long long value)
{
	memcpy(buffer(),&value,sizeof(unsigned long long));
}

void MemberInfo::set(float value)
{
	memcpy(buffer(),&value,sizeof(float));
}

void MemberInfo::set(double value)
{
	memcpy(buffer(),&value,sizeof(double));
}

void MemberInfo::set(long double value)
{
	memcpy(buffer(),&value,sizeof(long double));
}

bool MemberInfo::toBool()
{
	return *((bool*)buffer());
}

char MemberInfo::toChar()
{
	return *((char*)buffer());
}

unsigned char MemberInfo::toUChar()
{
	return *((unsigned char*)buffer());
}

short MemberInfo::toShort()
{
	return *((short*)buffer());
}

unsigned short MemberInfo::toUShort()
{
	return *((unsigned short*)buffer());
}

int MemberInfo::toInt()
{
	return *((int*)buffer());
}

unsigned int MemberInfo::toUInt()
{
	return *((unsigned int*)buffer());
}

long MemberInfo::toLong()
{
	return *((long*)buffer());
}

unsigned long MemberInfo::toULong()
{
	return *((unsigned long*)buffer());
}

long long MemberInfo::toLongLong()
{
	return *((long long*)buffer());
}

unsigned long long MemberInfo::toULongLong()
{
	return *((unsigned long long*)buffer());
}

float MemberInfo::toFloat()
{
	return *((float*)buffer());
}

double MemberInfo::toDouble()
{
	return *((double*)buffer());
}

long double MemberInfo::toLongDouble()
{
	return *((long double*)buffer());
}

XMLStruct::XMLStruct()
{
}

bool XMLStruct::read(QString xml)
{
	size(0);

	QFile file(xml);
	if(!file.open(QFile::ReadOnly|QFile::Text))
	{
		return false;
	}
	
	if(!dom().setContent(&file,true))
	{
		return false;
	}
	
	QDomNodeList list(dom().elementsByTagName("protocol"));
	for(int count = 0 ; count < list.size() ; ++count)
	{
		QDomElement element(list.at(count).toElement());
		QString name(element.attribute("name"));
		int id(element.attribute("id").toInt());

		StructureInfo info;
		info.name(name);

		QDomElement structure(element.elementsByTagName("structure").at(0).toElement());
		QDomNodeList datatypes(structure.childNodes());
		if(datatypes.size())
		{
			std::vector<MemberInfo*> ms;
			size_t s = 0;
			for(int typeCount = 0 ; typeCount < datatypes.size() ; ++typeCount)
			{
				QDomElement datatype(datatypes.at(typeCount).toElement());
				if(datatype.tagName()==TYPE_BOOL)
				{
					ADDMEMBER(bool,TYPE_BOOL);
				}
				else if(datatype.tagName()==TYPE_CHAR)
				{
					ADDMEMBER(char,TYPE_CHAR);
				}
				else if(datatype.tagName()==TYPE_UCHAR)
				{
					ADDMEMBER(unsigned char,TYPE_UCHAR);
				}
				else if(datatype.tagName()==TYPE_SHORT)
				{
					ADDMEMBER(short,TYPE_SHORT);
				}
				else if(datatype.tagName()==TYPE_USHORT)
				{
					ADDMEMBER(unsigned short,TYPE_USHORT);
				}
				else if(datatype.tagName()==TYPE_INT)
				{
					ADDMEMBER(int,TYPE_INT);
				}
				else if(datatype.tagName()==TYPE_UINT)
				{
					ADDMEMBER(unsigned int,TYPE_UINT);
				}
				else if(datatype.tagName()==TYPE_LONG)
				{
					ADDMEMBER(long,TYPE_LONG);
				}
				else if(datatype.tagName()==TYPE_ULONG)
				{
					ADDMEMBER(unsigned long,TYPE_ULONG);
				}
				else if(datatype.tagName()==TYPE_LONGLONG)
				{
					ADDMEMBER(long long,TYPE_LONGLONG);
				}
				else if(datatype.tagName()==TYPE_ULONGLONG)
				{
					ADDMEMBER(unsigned long long,TYPE_ULONGLONG);
				}
				else if(datatype.tagName()==TYPE_FLOAT)
				{
					ADDMEMBER(float,TYPE_FLOAT);
				}
				else if(datatype.tagName()==TYPE_DOUBLE)
				{
					ADDMEMBER(double,TYPE_DOUBLE);
				}
				else if(datatype.tagName()==TYPE_LONGDOUBLE)
				{
					ADDMEMBER(long double,TYPE_LONGDOUBLE);
				}
			}

			info.size(s);
			info.buffer(new char[s/sizeof(char)]);

			void* pos(info.buffer());

			for(size_t memberCount = 0 ; memberCount < ms.size() ; ++memberCount)
			{
				ms[memberCount]->buffer(pos);
				pos = (char*)pos+ms[memberCount]->size()/sizeof(char);
			}

			structures[id] = info;

			if(size()<s) size(s);
		}
	}

	return true;
}
